<?php


namespace App\Http\Controllers\Aguas;

use Carbon\Carbon;
use App\Traits\ApiResponse;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use App\Model\Ventanilla\Catalogo\CatDocumento;
use App\Model\Ventanilla\Catalogo\Estatus;
use App\Model\Ventanilla\Catalogo\Zona;
use App\Model\Ventanilla\Tramite\Cancela;
use App\Model\Ventanilla\Tramite\Correccion;
use App\Model\Ventanilla\Tramite\Documento;
use Barryvdh\DomPDF\Facade as PDF;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Model\Ventanilla\Tramite\Aguas;
use App\Http\Requests\CorreccionEnvioRequest;
use App\Http\Requests\CancelarRequest;

class NotarioController extends Controller
{
    use ApiResponse;
    /**
     * Get the path the user should be redirected to when they are not authenticated.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return string
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index(Request $request)
    {
        $tramite = Aguas::where('VENTANILLA', '=', '0')->where('FOLIONODO', '!=', "");
        if ($request->input('folio')) {
            $tramite = $tramite->where('FOLIONODO', 'like', '%'.$request->input('folio').'%');
        }
        if ($request->input('estatus') != "") {
            $estatus = Estatus::where('DESCRIPCION', 'like', '%'.$request->input('estatus').'%')->pluck('IDESTATUSTRAMITE');
            if (count($estatus)>0) {
                $tramite = $tramite->where('IDESTATUS', '=', $estatus[0]);
            }
        }
        if ($request->input('estatus') != "" || $request->input('folio') != "") {
            $tramite = $tramite->orderBy('IDTRAMITE', 'desc')->paginate(count(Aguas::all()));
        } else {
            $tramite = $tramite->orderBy('IDTRAMITE', 'desc')->paginate(10);
        }
        $notify = Aguas::where('VENTANILLA', '=', '0')->where('FOLIONODO', '!=', "")->orderBy('FECHAINICIO', 'desc')->get('FOLIONODO', 'IDESTATUS');
        return view('aguas/index', compact(['tramite','notify']));
    }

    public function buscarFolio(Request $request)
    {
        $tramite = Aguas::where('VENTANILLA', '=', '0');
        if ($request->input('folioNodo')) {
            $tramite = $tramite->where('IDTRAMITE', '=', $request->input('folioNodo'));
        }
        return view('aguas/index', compact(['tramite','notify']));
    }

    public function verNotificaciones()
    {
        $notify = Aguas::where('VENTANILLA', '=', '0')->where('FOLIONODO', '!=', "")->where('IDESTATUS', '=', 1)
                ->orWhere('IDESTATUS', '=', 4)->orderBy('FECHAACTUALIZACION', 'desc')->get();
        return $notify;
    }

    public function show(Request $request)
    {
        $id = $request->input('idTramite');
        $item = Aguas::find($id); // Get users from DB
        return $item;
    }

    public function verSolicitud(Request $request)
    {
        $conexionSei = new ConexionSei();
        $id = $request->input('idTramite');
        $item = Aguas::find($id); // Get users from DB
        $tipoDocumento = CatDocumento::where('DESCRIPCION', '=', 'SOLICITUD NOTARIOS')->pluck('IDDOCUMENTO');
        $documento = Documento::where('TIPODOCUMENTO', '=', $tipoDocumento[0])->where('FOLIOINTERNO', '=', $item->FOLIONODO)->get();
        if (count($documento) == 0) {
            $hash = $item->HASH;
            $data = array('hash'=>$hash,'departamento' => 'AGUAS');
            $url = $conexionSei->conexionSEI($data);

            $solicitud = new Documento;
            $solicitud->NOMBREARCHIVO = 'Solicitud.pdf';
            $solicitud->CONTENTTYPE = 'application/pdf';
            $solicitud->TIPODOCUMENTO = $tipoDocumento[0];
            $solicitud->FOLIOINTERNO = $item->FOLIONODO;
            $solicitud->BASE64 = base64_encode(file_get_contents($url));
            $solicitud->save();
            $response = array(
                'url' => base64_encode(file_get_contents($url))
            );
        } elseif (count($documento) == 1) {
            $response = array(
                'url' => $documento[0]['BASE64']
            );
        }

        return $response;
    }
    public function verCancelacion(Request $request)
    {
        $id = $request->input('idTramite');
        $item = Aguas::find($id); // Get users from DB
        $cancela = Cancela::where('FOLIOINTERNO', '=', $item->FOLIONODO)->pluck('ANOTACIONES');
        $response = array('MOTIVO' => $cancela[0]);
        return $response;
    }

    public function cancel(CancelarRequest $request)
    {
        $id = Auth::id();
        Log::debug('An informational message funcion correccion.');
        Log::info('User: '. $id);
        try {
            $motivo = $request->input('motivo');
            $tramite = Aguas::find($request->input('idtramite'));
            $folio = $tramite->FOLIONODO;
            Log::info('Folio tramite aguas: '. $folio);
            $data = array(
                'folio'=>$folio,
                'anotaciones' => $motivo,
                'estatus' =>  'CONCLUIDO_CANCELADO'
            );
            Log::info('Data para nodo cancelar: ');
            Log::info($data);
            //dd($data);
            $conexion = new ConexionNodo($data);
            $respuestaNodo = $conexion->enviarTramiteCanceladoNodo($data);
            Log::info('Respuesta de nodo cancelar: '. $respuestaNodo);
            Log::info($respuestaNodo);
            if ($respuestaNodo) {
                $tramite->IDESTATUS = 47;
                $tramite->FECHAACTUALIZACION = date('Y/m/d');
                $tramite->save();

                $cancela = new Cancela;
                $cancela->ANOTACIONES = $motivo;
                $cancela->FOLIOINTERNO = $folio;
                $cancela->save();
                Log::info('Proceso exitoso cancelacion: '. $folio);
                Log::info('Respuesta: '. $this->showSuccess($folio));
                return $this->showSuccess($tramite->FOLIONODO);
            } else {
                Log::error('Error al mandar a cancelacion');
                return $this->showErrorWeb('error', ['Error al mandar a cancelacion'], 500);
            }
        } catch (\Exception $exception) {
            Log::error($exception->getMessage());
            return $this->showErrorWeb('error', [
                $exception->getMessage()
            ], 500);
        }
    }

    public function correccion(CorreccionEnvioRequest $request)
    {
        $tramite = Aguas::find($request->input('idtramite'));
        $data = array(
            'folio' =>  $tramite->FOLIONODO,
            'anotaciones' => $request->input('motivo'),
            'estatus'=>$tramite->IDESTATUS,
        );
        Log::info('Data para nodo correccion: ');
        Log::info($data);
        //Envio de correccion a nodo
        $conexion = new ConexionNodo($data);
        $respuestaNodo = $conexion->enviarTramiteCorreccionNodo($data);
        if ($respuestaNodo) {
            //Actualizar tramite
            try {
                Aguas::where('FOLIONODO', $tramite->FOLIONODO)->first()->update(['IDESTATUS' => 50,'FECHAACTUALIZACION'=> Carbon::now()]);
                $correccion = new Correccion;
                $correccion->ANOTACIONES = $request->input('motivo');
                $correccion->FOLIOINTERNO = $tramite->FOLIONODO;
                $correccion->save();
                return $this->showSuccess($tramite->FOLIONODO);
            } catch (\Exception $exception) {
                Log::error($exception->getMessage());
                return $this->showErrorWeb('error', [
                $exception->getMessage()
            ], 500);
            }
        } else {
            return $this->showErrorWeb('error', ['Error al mandar a corrección'], 500);
        }
    }

    public function update(Request $request)
    {
        $tramite = Aguas::find($request->input('idTramite'));

        switch ($tramite->estatus->DESCRIPCION) {
            case 'Solicitud Nueva':
                $estatus = Estatus::where('DESCRIPCION', 'like', 'Solicitud Revisada')->pluck('IDESTATUSTRAMITE');
                $tramite->IDESTATUS = $estatus[0];
                $tramite->FECHAACTUALIZACION = date('Y/m/d');
                $tramite->save();
                break;
            case 'Tramite Pagado':
                $estatus = Estatus::where('DESCRIPCION', 'like', 'Generar Precedula')->pluck('IDESTATUSTRAMITE');
                $tramite->IDESTATUS = $estatus[0];
                $tramite->FECHAACTUALIZACION = date('Y/m/d');
                $tramite->save();
                break;
            case 'Correccion':
                $estatus = Estatus::where('DESCRIPCION', 'like', 'Solicitud Revisada')->pluck('IDESTATUSTRAMITE');
                $tramite->IDESTATUS = $estatus[0];
                $tramite->FECHAACTUALIZACION = date('Y/m/d');
                $tramite->save();
                break;
            default:
                break;
        }
        $respuesta = array('estatus'=> 'true');
        return $respuesta;
    }
}
