<?php

namespace App\Http\Controllers\Api;

use Carbon\Carbon;
use App\Traits\ApiResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Mail;
use App\Http\Controllers\Controller;
use App\Http\Requests\NotariosRequest;
use App\Http\Requests\LineaPagoRequest;
use App\Http\Requests\CorreccionRequest;
use App\Model\Ventanilla\Tramite\Documento;
use App\Model\Ventanilla\Catalogo\CatDocumento;
use App\Model\Ventanilla\Catalogo\Estatus;
use App\Model\Ventanilla\Catalogo\Zona;
use App\Model\Ventanilla\Tramite\Aguas;
use App\Models\User;

class NotariosApiController extends Controller
{
    use ApiResponse;

    public function store(NotariosRequest $request)
    {
        $id = Auth::id();
        Log::debug('An informational message.');
        Log::info('User: '. $id);
        $create = $request->validated();
        Log::info($create);
        try {
            //Tramite
            $zona = Zona::where('DESCRIPCION', '=', $request->input('ZONADESCRIPCION'))->pluck('IDZONA');
            Log::info('Id Zona: '. $zona[0]);
            if ($zona[0]) {
                $create ['IDZONA'] = $zona[0];
                $create ['IDESTATUS'] = 1;
                $create ['VENTANILLA'] = 0;
                $create ['FECHAINICIO'] = Carbon::now();
                $create ['FECHAACTUALIZACION'] = Carbon::now();
                $tramite = Aguas::create($create);
                Log::info('Id tramite aguas creado: '. $tramite->IDTRAMITE);
                if ($tramite) {
                    $folio = 'cna_'.date('d/m').'-'.$request->input('NUMERONOTARIO').$request->input('ZONADESCRIPCION').'-'.(($tramite->IDTRAMITE));
                    Log::info('Folio Creado: '. $folio);
                    $update = Aguas::where('IDTRAMITE', $tramite->IDTRAMITE)->first()->update(['FOLIOAGUAS' => $folio]);
                    $create ['FOLIOINTERNO'] = $folio;
                    //Archivos
                    foreach ($create['files'] as $key => $value) {
                        //Log::info('Documento key: '. $key);
                        //Log::info('Documento value: ');
                        //Log::info($value);
                        $documento = Documento::create($value);
                        Log::info('Documento id creado: '. $documento->IDDOCUMENTO);
                    }
                    if ($update) {
                        Log::info('Proceso exitoso: '. $folio);
                        Log::info('Respuesta: '. $this->showSuccess($folio));
                        //dd($update);
                        //Send Email-----------------------------------------------
                        /*  Mail::send(
                             'email.notification',
                             ['create'=>$create],
                             function ($m) use ($create) {
                                 $m->to('jesuschicho@zurco.com.mx')
                             ->subject('Notificacion sistemas aguas');
                             }
                         ); */
                        return $this->showSuccess($folio);
                    } else {
                        return $this->showError('error', ['Error al guardar tramite'], 500);
                    }
                } else {
                    return $this->showError('error', ['Error al guardar tramite'], 500);
                }
            } else {
                return $this->showError('error', ['Zona no existente'], 500);
            }
        } catch (\Exception $exception) {
            Log::error($exception->getMessage());
            return $this->showError('error', [
                $exception->getMessage()
            ], 500);
        }
    }


    public function lineaPago(LineaPagoRequest $request)
    {
        Log::info('Peticion de lineaPago');
        $folio = $request->input('FOLIONODO');
        $tramite =Aguas::where('FOLIONODO', $request->only('FOLIONODO'))->first();
        if ($tramite) {
            Log::info('Tramite encontrado');
            $tipoDocumento = CatDocumento::where('DESCRIPCION', 'like', 'COMPROBANTE PAGO NOTARIO')->pluck('IDDOCUMENTO');
            try {
                $create = $request->validated();
                //Archivo
                foreach ($create as $key => $value) {
                    if ($key==='comprobantePago') {
                        $value ['TIPODOCUMENTO'] = $tipoDocumento[0];
                        Documento::create($value);
                        break;
                    }
                }
                //Tramite
                $estatus = Estatus::where('DESCRIPCION', 'like', 'Tramite Pagado')->pluck('IDESTATUSTRAMITE');
                $update = Aguas::where('IDTRAMITE', $tramite->IDTRAMITE)->first()->update(['IDESTATUS' => $estatus[0],'FECHAACTUALIZACION'=> Carbon::now()]);
                Log::info('Proceso exitoso linea: '. $folio);
                Log::info('Respuesta: '. $this->showSuccess($folio));
                return $this->showSuccess($tramite->FOLIOAGUAS);
            } catch (\Exception $exception) {
                Log::error($exception->getMessage());
                return $this->showError('error', [
                $exception->getMessage()
            ], 500);
            }
        } else {
            return $this->showError('error', ['Tramite no existente'], 500);
        }
    }

    public function correccion(CorreccionRequest $request)
    {
        $id = Auth::id();
        Log::debug('An informational message funcion correccion.');
        Log::info('User: '. $id);
        $data = $request->validated();
        Log::debug($data);
        $folio = $request->input('FOLIONODO');

        try {
            //Archivos
            foreach ($data['files'] as $key => $value) {
                Log::info('Documento key: '. $key);
                Log::info('Documento value: ');
                Log::info($value);
                $update = Documento::where('FOLIOINTERNO', $folio)
                      ->where('TIPODOCUMENTO', $value['TIPODOCUMENTO'])
                      ->first()
                      ->update(['NOMBREARCHIVO' =>  $value['NOMBREARCHIVO'],'PATH'=> $value['PATH']]);
                Log::info('Documento id de Nodo - actualizado: '. $value['IDDOCUMENTO']);
            }
            //Tramite
            $estatus = Estatus::where('DESCRIPCION', 'like', 'Correccion')->pluck('IDESTATUSTRAMITE');
            $update = Aguas::where('FOLIONODO', $folio)->first()->update(
                [
                    'CATASTROCLAVEAGUA' => $request->input('CATASTROCLAVEAGUA'),
                    'CATASTRONUMERO' => $request->input('CATASTRONUMERO'),
                    'NOMBRE' => $request->input('NOMBRE'),
                    'APELLIDOPATERNO' => $request->input('APELLIDOPATERNO'),
                    'APELLIDOMATERNO' => $request->input('APELLIDOMATERNO'),
                    'NOMBRESUSCRITO' => $request->input('NOMBRESUSCRITO'),
                    'APELLIDOPATERNOSUSCRITO' => $request->input('APELLIDOPATERNOSUSCRITO'),
                    'APELLIDOMATERNOSUSCRITO' => $request->input('APELLIDOMATERNOSUSCRITO'),
                    'CALLE' => $request->input('CALLE'),
                    'NUMEROEXTERIOR' => $request->input('NUMEROEXTERIOR'),
                    'NUMEROINTERIOR' => $request->input('NUMEROINTERIOR'),
                    'FRACCIONAMIENTO' => $request->input('FRACCIONAMIENTO'),
                    'COLONIA' => $request->input('COLONIA'),
                    'LOCALIDAD' => $request->input('LOCALIDAD'),
                    'TELEFONO' => $request->input('TELEFONO'),
                    'NUMERONOTARIO' => $request->input('NUMERONOTARIO'),
                    'NOMBRENOTARIO' => $request->input('NOMBRENOTARIO'),
                    'HASH' => $request->input('HASH'),
                    'IDESTATUS' => $estatus[0],
                    'FECHAACTUALIZACION'=> Carbon::now()
                ]
            );
            Log::info('Proceso exitoso: '. $folio);
            Log::info('Respuesta: '. $this->showSuccess($folio));
            return $this->showSuccess($folio);
        } catch (\Exception $exception) {
            Log::error($exception->getMessage());
            return $this->showError('error', [
                $exception->getMessage()
            ], 500);
        }
    }
}
