<?php

namespace App\Http\Controllers;

use Carbon\Carbon;
use App\Traits\ApiResponse;
use Illuminate\Http\Request;
use Barryvdh\DomPDF\Facade as PDF;
use App\Http\Controllers\Controller;
use App\Model\Ventanilla\Tramite\Aguas;
use App\Model\Ventanilla\Catalogo\Estatus;
use App\Model\Ventanilla\Tramite\Documento;
use App\Model\Ventanilla\Catalogo\CatDocumento;
use App\Http\Requests\CorreccionEnvioRequest;
use App\Http\Requests\PrecedulaRequest;
use App\Http\Requests\LineaRequest;
use Illuminate\Support\Facades\Storage;

use Kreait\Firebase;
use Kreait\Firebase\ServiceAccount;
use Google\Cloud\Firestore\FirestoreClient;
use Kreait\Firebase\Auth as FirebaseAuth;
use Kreait\Firebase\Auth\SignInResult\SignInResult;
use Kreait\Firebase\Exception\FirebaseException;
use Kreait\Firebase\Contract\Firestore;

use App\Http\Controllers\Aguas\ConexionNodo;
use App\Http\Controllers\Aguas\ConexionSei;

class ArchivosController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    use ApiResponse;

    public function generarCertificado($idTramite)
    {
        try {
            $tramite = Aguas::find($idTramite);
            $conexionSei = new ConexionSei();
            $hash = $conexionSei->hash($tramite);
            if ($hash) {
                $data = array('hash'=>$hash,'departamento' => 'AGUAS');
                $url = $conexionSei->conexionSEI($data);
                //$url = 'https://firebasestorage.googleapis.com/v0/b/tesoreria-front-dev.appspot.com/o/doc_prueba_test_2.pdf?alt=media&token=7e3404b6-9b59-49de-bf40-763cdc1ca14d';
                if ($url) {
                    unlink('assets/media/files/ReporteAguas_V3.1.xml');
                    //firebase
                    $uploadedfile = file_get_contents($url);
                    app('firebase.storage')->getBucket()->upload($uploadedfile, ['name' => 'aguas/files/'.$tramite->IDTRAMITE.'/certificado.pdf']);
                    $pathReference = app('firebase.storage')->getBucket()->object('aguas/files/'.$tramite->IDTRAMITE.'/certificado.pdf');
                    $expiresAt = Carbon::now()->addDays(7);
                    if ($pathReference->exists()) {
                        $path = $pathReference->signedUrl($expiresAt);
                        //dd($path);
                        $data = array(
                            'urlfirebase' => $path,
                            'folio' => $tramite->FOLIONODO,
                            'url'=>$url,
                        );
                        //Envio de certificado - tramite terminado
                        $conexion = new ConexionNodo($data);
                        $respuesta = $conexion->enviarTramiteTerminadoNodo($data);
                        if ($respuesta) {
                            $tipoDocumento = CatDocumento::where('DESCRIPCION', 'like', 'CERTIFICADO FIRMADO')->pluck('IDDOCUMENTO');
                            $documento = new Documento;
                            $documento->NOMBREARCHIVO = 'certificado.pdf';
                            $documento->CONTENTTYPE = 'application/pdf';
                            $documento->PATH = $path;
                            $documento->BASE64 = null;
                            $documento->TIPODOCUMENTO = $tipoDocumento[0];
                            $documento->FOLIOINTERNO = $tramite->FOLIONODO;
                            $documento->save();
                            $estatus = Estatus::where('DESCRIPCION', 'like', 'Certificado Emitido')->pluck('IDESTATUSTRAMITE');
                            $tramite->IDESTATUS = $estatus[0];
                            $tramite->FECHAACTUALIZACION = date('Y/m/d');
                            $tramite->save();
                            return $this->showSuccess($tramite->FOLIONODO);
                        } else {
                            return $this->showErrorWeb('error', ['Error al mandar a nodo'], 500);
                        }
                    }
                } else {
                    return $this->showErrorWeb('error', ['Error al mandar al obtener url'], 500);
                }
            } else {
                return $this->showErrorWeb('error', ['Error al mandar al generar hash'], 500);
            }
        } catch (\Exception $exception) {
            return $this->showErrorWeb('error', [
                $exception->getMessage()
            ], 500);
        }
    }

    public function generaPrecedula(PrecedulaRequest $request, $id)
    {
        try {
            $tramite = Aguas::find($id);
            $respuesta = $this->generaPrecedulaFile($tramite);
            if ($respuesta) {
                $estatus = Estatus::where('DESCRIPCION', 'like', 'Ver Precedula')->pluck('IDESTATUSTRAMITE');
                $tramite->BIMESTRE = $request->input('bimestre');
                $tramite->ANIOBIMESTRE = $request->input('aniobimetre');
                $tramite->RECIBOOFICIAL = $request->input('recibo');
                $tramite->FECHARECIBO = $request->input('fecharecibo');
                $tramite->PAGODERECHOS = $request->input('pagoderechos');
                $tramite->FECHAACTUALIZACION = date('Y/m/d');
                $tramite->IDESTATUS = $estatus[0];
                $tramite->FECHADERECHO = $request->input('fechaderechos');
                $tramite->save();
                return $this->showSuccess($tramite->FOLIONODO);
            } else {
                return $this->showErrorWeb('error', ['Error al mandar al generar'], 500);
            }
        } catch (\Exception $exception) {
            return $this->showErrorWeb('error', [
                $exception->getMessage()
            ], 500);
        }
    }

    public function generaPrecedulaFile($tramite)
    {
        try {
            $fecharecibo = explode(" ", $tramite->FECHARECIBO);
            $fechaderecho = explode(" ", $tramite->FECHADERECHO);
            $data = ['tramite' => $tramite, 'FechaActual' => Carbon::now()->format('Y-m-d'),
                    'fecharecibo'=>$fecharecibo[0],'fechaderecho'=>$fechaderecho[0]];
            $pdf = PDF::loadView('pdf/precedula', $data);
            $pdf->save('assets/media/files/precedula.pdf');
            //Firebase
            $uploadedfile = fopen('assets/media/files/precedula.pdf', 'r');
            //dd($uploadedfile);
            app('firebase.storage')->getBucket()->upload($uploadedfile, ['name' => 'aguas/files/'.$tramite->IDTRAMITE.'/precedula.pdf']);
            unlink('assets/media/files/precedula.pdf');
            $pathReference = app('firebase.storage')->getBucket()->object('aguas/files/'.$tramite->IDTRAMITE.'/precedula.pdf');
            $expiresAt = Carbon::now()->addDays(7);
            if ($pathReference->exists()) {
                $path = $pathReference->signedUrl($expiresAt);
                //dd($path);
                $tipoDocumento = CatDocumento::where('DESCRIPCION', 'like', 'Precedula')->pluck('IDDOCUMENTO');
                $documento = new Documento;
                $documento->NOMBREARCHIVO = 'precertificado.pdf';
                $documento->CONTENTTYPE = 'application/pdf';
                $documento->PATH = $path;
                $documento->BASE64 = null;
                $documento->TIPODOCUMENTO = $tipoDocumento[0];
                $documento->FOLIOINTERNO = $tramite->FOLIONODO;
                $documento->save();
                return true;
            } else {
                return false;
            }
        } catch (\Exception $exception) {
            return $this->showErrorWeb('error', [
            $exception->getMessage()
        ], 500);
        }
    }

    public function generarLinea(LineaRequest $request, $id)
    {
        $url = $request->input('url');
        $motivo = $request->input('motivo');
        //return $this->showSuccess('folio');
        //return $this->showErrorWeb('error', ['Error al enviar linea de captura'], 500);
        //return $this->showSuccess($request->input('motivo').' '.$request->input('url'));
        try {
            $tramite = Aguas::find($id);
            $respuesta = $this->envioLineaNodo($tramite, $motivo, $url);
            //dd($respuesta);
            if ($respuesta) {
                $estatus = Estatus::where('DESCRIPCION', 'like', 'Linea de Pago Emitida')->pluck('IDESTATUSTRAMITE');
                $tramite->IDESTATUS = $estatus[0];
                $tramite->FECHAACTUALIZACION = date('Y/m/d');
                $tramite->save();
                return $this->showSuccess($tramite->FOLIONODO);
            } else {
                return $this->showErrorWeb('error', ['Error al enviar linea de captura'], 500);
            }
        } catch (\Exception $exception) {
            return $this->showErrorWeb('error', [
            $exception->getMessage()
        ], 500);
        }
    }

    public function envioLineaNodo($tramite, $motivo, $url)
    {
        try {
            $data = array(
                'urlLineaCaptura' => 'sin url de linea',
                'folio' => $tramite->FOLIONODO,
                'lineaCaptura'=>$url,
                'motivo'=>$motivo
            );
            //Enviar a nodo
            $conexion = new ConexionNodo($data);
            $respuestaNodo = $conexion->enviarLineaCapturaNodo($data);

            if ($respuestaNodo) {
                //dd($respuestaNodo);
                $tipoDocumento = CatDocumento::where('DESCRIPCION', 'like', 'LINEA DE PAGO')->pluck('IDDOCUMENTO');
                $documento = new Documento;
                $documento->NOMBREARCHIVO = 'lineaCaptura.pdf';
                $documento->CONTENTTYPE = 'application/pdf';
                $documento->PATH = $url;
                $documento->BASE64 = null;
                $documento->TIPODOCUMENTO = $tipoDocumento[0];
                $documento->FOLIOINTERNO = $tramite->FOLIONODO;
                $documento->save();
                return $respuestaNodo;
            } else {
                return $this->showErrorWeb('error', ['Error al enviar linea de captura nodo'], 500);
            }
        } catch (\Exception $exception) {
            return $this->showErrorWeb('error', [
                $exception->getMessage()
            ], 500);
        }
    }
}
